import unittest
from datetime import date

import jinsi


class DatesTest(unittest.TestCase):
    def test_date(self):
        result = jinsi.load1s("""
        value: 2020-02-20
        """)
        self.assertEqual(result['value'], date(2020, 2, 20))


if __name__ == '__main__':
    unittest.main()



from .common import JinsiTestCase


class JinsiEach(JinsiTestCase):

    def test_each(self):
        doc = """\
            ::let:
              qux:
                - a
                - b
                - c
            foo:
              ::merge:
                ::each qux as name:
                  ::object:
                    - <<name>>_key
                    - <<name>>_value
        """

        expected = {
            'foo': {
                'a_key': 'a_value',
                'b_key': 'b_value',
                'c_key': 'c_value',
            }
        }

        self.check(expected, doc)

    def test_each_with_dynamic_loop_parameter(self):
        doc = """\
            resources:
              ::call resources_template:
                $rs:
                  - one
                  - two
            ::let:
              resource:
                ::object:
                  - <<$res.name>>_key
                  - name: <<$res.name>>_value
              resources_template:
                ::merge:
                    ::each $rs as $r:
                      - ::call resource:
                          $res:
                            name: <<$r>>
                      - ::call resource:
                          $res:
                            name: <<$r>>_foo
                      - ::call resource:
                          $res:
                            name: <<$r>>_bar
        """

        expected = {
            "resources": {
                "one_key": {
                    "name": "one_value"
                },
                "one_foo_key": {
                    "name": "one_foo_value"
                },
                "one_bar_key": {
                    "name": "one_bar_value"
                },
                "two_key": {
                    "name": "two_value"
                },
                "two_foo_key": {
                    "name": "two_foo_value"
                },
                "two_bar_key": {
                    "name": "two_bar_value"
                },
            }
        }

        self.check(expected, doc)


    def test_each_with_lexicographic_loop_parameter(self):
        doc = """\
            resources:
              ::call resources_template:
                $rs:
                  - one
                  - two
            ::let:
              resource:
                ::object:
                  - <<$res.name>>_key
                  - name: <<$res.name>>_value
              resources_template:
                ::merge:
                    ::each $rs as r:
                      - ::call resource:
                          $res:
                            name: <<r>>
                      - ::call resource:
                          $res:
                            name: <<r>>_foo
                      - ::call resource:
                          $res:
                            name: <<r>>_bar
        """

        expected = {
            "resources": {
                "one_key": {
                    "name": "one_value"
                },
                "one_foo_key": {
                    "name": "one_foo_value"
                },
                "one_bar_key": {
                    "name": "one_bar_value"
                },
                "two_key": {
                    "name": "two_value"
                },
                "two_foo_key": {
                    "name": "two_foo_value"
                },
                "two_bar_key": {
                    "name": "two_bar_value"
                },
            }
        }

        self.check(expected, doc)




import json
import unittest
from decimal import Decimal

import yaml

from jinsi import *
from jinsi.yamlutil import Loader


class JinsiTestCase(unittest.TestCase):

    def check(self, expected, doc: str, dezimal_foo: bool = False, args: dict = None):
        rendered = render1s(doc, as_json=True, args=args)
        if dezimal_foo:
            self.assertEqual(expected, json.loads(rendered, parse_int=Decimal, parse_float=Decimal))
        else:
            self.assertEqual(expected, json.loads(rendered))

        rendered = render1s(doc, as_json=False, args=args)
        if dezimal_foo:
            self.assertEqual(expected, yaml.load(rendered, Loader=Loader))
        else:
            self.assertEqual(expected, yaml.safe_load(rendered))

        if dezimal_foo:
            loaded = load1s(doc, numtype=Decimal, args=args)
        else:
            loaded = load1s(doc, args=args)
        self.assertEqual(expected, loaded)



import unittest

from .common import JinsiTestCase


class JinsiExamples(JinsiTestCase):

    def test_flatten(self):
        doc = """\
            ::let:
                xs:
                    - - 1
                    - - 2
                    - - 3
            xs:
                ::flatten:
                    ::get: xs
        """

        expected = {
            'xs': [1, 2, 3]
        }

        self.check(expected, doc)

    def test_select_from_list(self):
        doc = """\
            ::let:
                xs:
                    - - 1
                    - - 2
                    - - 3
            xs:
                ::select:
                    - ::get: xs
                    - 1
        """

        expected = {
            'xs': [2]
        }

        self.check(expected, doc)

    def test_select_from_object(self):
        doc = """\
            ::let:
                xs:
                    foo: 1
                    bar: 2
            xs:
                ::select:
                    - ::get: xs
                    - bar
        """

        expected = {
            'xs': 2
        }

        self.check(expected, doc)

    def test_select_range_from_list(self):
        doc = """\
            ::let:
                xs:
                    - 1
                    - 2
                    - 3
            xs:
                ::select:
                    - ::get: xs
                    - 0
                    - 2
        """

        expected = {
            'xs': [1, 2]
        }

        self.check(expected, doc)

    def test_select_range_from_string(self):
        doc = """\
            ::let:
                xs: "abc"
            xs:
                ::select:
                    - ::get: xs
                    - 0
                    - 2
        """

        expected = {
            'xs': "ab"
        }

        self.check(expected, doc)

    def test_sha256(self):
        doc = """\
            ::let:
                v: "value"
            xs:
                ::sha256:
                - ::get: v
        """

        expected = {
            'xs': 'cd42404d52ad55ccfa9aca4adc828aa5800ad9d385a0671fbcbf724118320619'
        }

        self.check(expected, doc)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from decimal import Decimal

from .common import JinsiTestCase


class JinsiExamples(JinsiTestCase):

    def test_simple_template(self):
        doc = """\
            ::let:
              user:
                ::object:
                  - ::titlecase:
                      ::get: $user.username
                  - Type: AWS::IAM::User
                    Properties:
                      UserName:
                        ::get: $user.username
                      Groups:
                        - Administrators
                      LoginProfile:
                        Password:
                          ::get: $user.password
                          ::else: default
                        PasswordResetRequired: Yes
              users:
                ::merge:
                  ::each $ as $user:
                    ::call user:
            
            Resources:
              ::call users:
                - username: jim
                  password: one
                - username: jack
                  password: two
                - username: johnny
        """

        expected = {
            "Resources": {
                "Jim": {
                    "Type": "AWS::IAM::User",
                    "Properties": {
                        "UserName": "jim",
                        "Groups": ["Administrators"],
                        "LoginProfile": {
                            "Password": "one",
                            "PasswordResetRequired": True
                        }
                    }
                },
                "Jack": {
                    "Type": "AWS::IAM::User",
                    "Properties": {
                        "UserName": "jack",
                        "Groups": ["Administrators"],
                        "LoginProfile": {
                            "Password": "two",
                            "PasswordResetRequired": True
                        }
                    }
                },
                "Johnny": {
                    "Type": "AWS::IAM::User",
                    "Properties": {
                        "UserName": "johnny",
                        "Groups": ["Administrators"],
                        "LoginProfile": {
                            "Password": "default",
                            "PasswordResetRequired": True
                        }
                    }
                }
            }
        }

        self.check(expected, doc)

    def test_example_0(self):
        doc = """\
            ::let:
              user:
                ::object:
                  - ::titlecase:
                      ::get: $user.username
                  - Type: AWS::IAM::User
                    Properties:
                      UserName:
                        ::get: $user.username
                      Groups:
                        - Administrators
                      LoginProfile:
                        Password:
                          ::get: $user.password
                          ::else: default
                        PasswordResetRequired: Yes
              users:
                ::merge:
                  ::each $ as $user:
                    ::call user:
            
            Resources:
              ::call users:
                - username: jim
                  password: one
                - username: jack
                  password: two
                - username: johnny
        """

        expected = {
            'Resources': {
                'Jim': {
                    'Type': 'AWS::IAM::User',
                    'Properties': {
                        'UserName': 'jim', 'Groups': ['Administrators'],
                        'LoginProfile': {
                            'Password': 'one',
                            'PasswordResetRequired': True
                        }}},
                'Jack': {
                    'Type': 'AWS::IAM::User',
                    'Properties': {
                        'UserName': 'jack', 'Groups': ['Administrators'],
                        'LoginProfile': {
                            'Password': 'two',
                            'PasswordResetRequired': True
                        }}},
                'Johnny': {
                    'Type': 'AWS::IAM::User',
                    'Properties': {
                        'UserName': 'johnny', 'Groups': ['Administrators'],
                        'LoginProfile': {
                            'Password': 'default',
                            'PasswordResetRequired': True
                        }}}
            }
        }

        self.check(expected, doc)

    def test_example_1(self):
        doc = """\
            ::let:
              name: XMLRpcParser_Main
            
            docs:
              - ::snakecase: <<name>>
              - ::uppercase:
                  ::snakecase:
                    ::get: name
              - ::uppercase:
                  ::kebabcase:
                    ::get: name
              - ::kebabcase:
                  ::get: name
              - ::titlecase:
                  ::get: name
              - ::camelcase:
                  ::get: name
              - ::titlecase: XmlRPCProcessor_23
              - ::div:
                  - 1
                  - 7
              - ::div:
                  - 355
                  - 113
              - ::div:
                  - 2.7
                  - 3.01
        """

        expected = {
            'docs': [
                'xml_rpc_parser_main',
                'XML_RPC_PARSER_MAIN',
                'XML-RPC-PARSER-MAIN',
                'xml-rpc-parser-main',
                'XmlRpcParserMain',
                'xmlRpcParserMain',
                'XmlRpcProcessor23',
                Decimal('0.1428571428571428571428571429'),
                Decimal('3.141592920353982300884955752'),
                Decimal('0.8970099667774086378737541528')
            ]
        }

        self.check(expected, doc, dezimal_foo=True)

    def test_example_2(self):
        doc = """\
            ::let:
              x: foo
              y: bar
              $x: qux
              $y: quuz
            
              template:
                - ::get: x
                - ::get: y
                - ::get: $x
                - ::get: $y
                - ::get: $z
            
            formatted: hello <<x>> woohoo <<y>> yeah <<$x>> and <<$y>>
            
            cool:
              some-<<x>>: <<y>>
              woohoo: <<x>>/<<y>>
            
            list:
              - ::get: x
              - ::get: y
              - ::get: $x
              - ::get: $y
              - ::get: JINSI_TEST_SHELL
              - ::get: JINSI_TEST_HOSTNAME
                ::else: unknown
            
            x:
              ::let:
                something:
                  ::get: $m
              y:
                - ::get: something
                  ::let:
                    $m: 3
                  ::else:
                    ::get: $z
                - ::get: $q
                  ::else: All okay.
            
            applied:
              ::let:
                x: keyfoo
                y: keybar
                $x: keyqux
                $y: keyquuz
                $z: zeeee
            
              ::call template:
                $y: callquuz
            
            applied2:
              ::call: template
        """

        expected = {
            'formatted': 'hello foo woohoo bar yeah qux and quuz',
            'cool': {
                'some-foo': 'bar',
                'woohoo': 'foo/bar',
            },
            'list': [
                'foo',
                'bar',
                'qux',
                'quuz',
                '/bin/hash',
                'unknown',
            ],
            'x': {
                'y': [
                    3.0,
                    'All okay.',
                ]
            },
            'applied': [
                'foo',
                'bar',
                'keyqux',
                'callquuz',
                'zeeee',
            ],
            'applied2': [
                'foo',
                'bar',
                'qux',
                'quuz',
                'zzz',
            ],
        }

        os.environ['JINSI_TEST_SHELL'] = '/bin/hash'

        self.check(expected, doc, args={'z': 'zzz'})

    def test_example_3(self):
        doc = """\
            ::let:
              users:
                ::each $ as $account:
                  ::object:
                    - ::get: $account
                    - Properties:
                        Some: props
            
              robousers:
                ::each $ as $account:
                  ::object:
                    - ::get: $account
                    - Properties:
                        Some: props
            
            Resources:
              ::call users:
                - jane
                - john
              ::call robousers:
                - ava
                - hal
              a: b
              c: d
        """

        expected = {
            'Resources': {
                'jane': {
                    'Properties': {
                        'Some': 'props'
                    }
                },
                'john': {
                    'Properties': {
                        'Some': 'props'
                    }
                }, 'ava': {
                    'Properties': {
                        'Some': 'props'
                    }
                }, 'hal': {
                    'Properties': {
                        'Some': 'props'
                    }
                },
                'a': 'b',
                'c': 'd',
            }
        }

        self.check(expected, doc)

    def test_example_4(self):
        doc = """\
            ::let:
              xs:
                - 1
                - 2
                - 7
                - 3
                - 2
                - 9
                - 4
                - 2
                - 5
            
            doc:
              ys:
                ::get: xs
              zs:
                ::let:
                  x:
                    ::explode:
                      - " "
                      - hello world out there
                ::get: x
              qs:
                ::let:
                  xs:
                    ::explode:
                      - " "
                      - hello out there
                ::each xs as x:
                  ::get: x
        """

        expected = {
            'doc': {
                'ys': [1, 2, 7, 3, 2, 9, 4, 2, 5],
                'zs': ['hello', 'world', 'out', 'there'],
                'qs': ['hello', 'out', 'there'],
            }
        }

        self.check(expected, doc)

    def test_example_5(self):
        doc = """\
            ::let:
              fib:
                ::when:
                  ::get: $n == 0 or $n == 1
                ::then:
                  ::get: $n
                ::else:
                  ::add:
                    - ::call fib:
                        $n:
                          ::get: $n - 1
                    - ::call fib:
                        $n:
                          ::get: $n - 2
              fibs:
                ::range_exclusive:
                  - 0
                  - ::get: $max
                    ::else: 10
            
            result:
              ::each fibs as $n:
                ::call: fib
        """

        expected = {
            'result': [
                0,
                1,
                1,
                2,
                3,
                5,
                8,
                13,
                21,
                34,
            ]
        }

        self.check(expected, doc)

    def test_example_6(self):
        doc = """\
            result:
              ::any:
                - ::when: false
                  ::then: 1
                - ::when: false
                  ::then: 2
                - 7
            
            result2:
              ::any:
                - []
                - {}
                - false
                - null
                - 1
                - 2
                - 3
        """

        expected = {
            'result': 7,
            'result2': 1,
        }

        self.check(expected, doc)


if __name__ == '__main__':
    unittest.main()



import re
import textwrap
import unittest
import io
from typing import Dict

from jinsi.main import main as jinsi_main


def capture(into: io.StringIO):
    def _print(arg, end='\n'):
        into.write(arg)
        into.write(end)

    return _print


def provide(dct: Dict[str, str]):
    def _open(arg):
        val = dct[arg]
        return io.StringIO(textwrap.dedent(val))

    return _open


class MainTest(unittest.TestCase):

    def test_version(self):
        res = io.StringIO()
        jinsi_main("--version", _print=capture(res), _open=provide({}), _stdin=io.StringIO(""))
        self.assertRegex(res.getvalue(), re.compile(r"0\.[1-9][0-9]*\.[0-9]"))

    def test_single_json_object_as_yaml(self):
        res = io.StringIO()
        jinsi_main("-", _print=capture(res), _open=provide({}), _stdin=io.StringIO("""{"x":3}"""))
        self.assertEqual("x: 3\n", res.getvalue())

    def test_multi_json_object_as_yaml(self):
        res = io.StringIO()
        jinsi_main("-", _print=capture(res), _open=provide({}), _stdin=io.StringIO("""{"x":3}\n{"y":3}\n"""))
        self.assertEqual("x: 3\n---\ny: 3\n", res.getvalue())

    def test_single_yaml_object_as_yaml(self):
        res = io.StringIO()
        jinsi_main("-", _print=capture(res), _open=provide({}), _stdin=io.StringIO("x: 3"))
        self.assertEqual("x: 3\n", res.getvalue())

    def test_multi_yaml_object_as_yaml(self):
        res = io.StringIO()
        jinsi_main("-", _print=capture(res), _open=provide({}), _stdin=io.StringIO("x: 3\n---\ny: 3\n"))
        self.assertEqual("x: 3\n---\ny: 3\n", res.getvalue())

    def test_single_json_object_as_json(self):
        res = io.StringIO()
        jinsi_main("-j", "-", _print=capture(res), _open=provide({}), _stdin=io.StringIO("""{"x":3}"""))
        self.assertEqual("""{"x":3}\n""", res.getvalue())

    def test_multi_json_object_as_json(self):
        res = io.StringIO()
        jinsi_main("-j", "-", _print=capture(res), _open=provide({}), _stdin=io.StringIO("""{"x":3}\n{"y":3}\n"""))
        self.assertEqual("""{"x":3}\n{"y":3}\n""", res.getvalue())

    def test_single_yaml_object_as_json(self):
        res = io.StringIO()
        jinsi_main("-j", "-", _print=capture(res), _open=provide({}), _stdin=io.StringIO("x: 3"))
        self.assertEqual("""{"x":3}\n""", res.getvalue())

    def test_multi_yaml_object_as_json(self):
        res = io.StringIO()
        jinsi_main("-j", "-", _print=capture(res), _open=provide({}), _stdin=io.StringIO("x: 3\n---\ny: 3\n"))
        self.assertEqual("""{"x":3}\n{"y":3}\n""", res.getvalue())


if __name__ == '__main__':
    unittest.main()



import unittest

from apm import *
from decimal import *

from jinsi import *


class LoadTest(unittest.TestCase):

    def test_load(self):
        doc = load1s("""\
            foo: bar
            qux: quux
            """)
        self.assertTrue(match(doc, Strict(dict(
            foo='bar',
            qux='quux',
        ))))

    def test_load_dezimal(self):
        doc = load1s("""\
            foo: 1.34
            """, numtype=Decimal)
        self.assertFalse(match(doc, Strict(dict(
            foo=Strict(float("1.34"))
        ))))
        self.assertTrue(match(doc, Strict(dict(
            foo=Strict(Decimal("1.34"))
        ))))

    def test_load_float(self):
        doc = load1s("""\
            foo: 1.34
            """)
        self.assertFalse(match(doc, Strict(dict(
            foo=Strict(Decimal("1.34"))
        ))))
        self.assertTrue(match(doc, Strict(dict(
            foo=Strict(float("1.34"))
        ))))

    def test_load_verbatim_and_ignore(self):
        doc = load1s("""\
            foo:
              ::verbatim:
                some text
            ::ignore:
              some shit
        """)
        self.assertEqual({'foo': 'some text'}, doc)

    def test_load_verbatim_object(self):
        doc = load1s("""\
            ::let:
                x: 'x'
            foo:
              qu<<x>>: quuz
            ::ignore:
              some shit
        """)
        self.assertEqual({'foo': {'qux': 'quuz'}}, doc)
        doc = load1s("""\
            ::let:
                x: 'x'
            foo:
              ::verbatim:
                qu<<x>>: quuz
            ::ignore:
              some shit
        """)
        self.assertEqual({'foo': {'qu<<x>>': 'quuz'}}, doc)

    def test_load_format_path(self):
        doc = load1s("""\
            ::let:
                resource:
                    name: some-resource
                    uuid: F3E835D1-9BCD-4D51-BA75-C90671E3B73E
                $res:
                    name: some-other-resource
                    uuid: 0CD49732-28AD-4437-8968-860DE4D84306
            test: |
                <<resource.name>> (<<resource.uuid>>)
                <<$res.name>> (<<$res.uuid>>)
        """)
        self.assertEqual({
            "test": "some-resource (F3E835D1-9BCD-4D51-BA75-C90671E3B73E)\n"
                    "some-other-resource (0CD49732-28AD-4437-8968-860DE4D84306)\n"
        }, doc)

    def test_string(self):
        doc = load1s("""\
            foo:
                ::let:
                    x: 1
                ::string: hello <<x>>
        """)
        self.assertEqual({"foo": "hello 1"}, doc)


if __name__ == '__main__':
    unittest.main()
